import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ThemeProvider extends ChangeNotifier {
  bool _isDarkMode = false;
  bool get isDarkMode => _isDarkMode;

  Color _primaryColor = Colors.indigo;
  Color get primaryColor => _primaryColor;

  Color _obligatoryColor = Colors.red.withOpacity(0.5);
  Color get obligatoryColor => _obligatoryColor;

  Color _facultativeColor = Colors.amber.withOpacity(0.5);
  Color get facultativeColor => _facultativeColor;

  Color _nonColor = Colors.grey.withOpacity(0.5);
  Color get nonColor => _nonColor;

  Color _crimeColor = Colors.red.withOpacity(0.5);
  Color get crimeColor => _crimeColor;

  Color _delitColor = Colors.amber.withOpacity(0.5);
  Color get delitColor => _delitColor;

  Color _contraventionColor = Colors.green.withOpacity(0.5);
  Color get contraventionColor => _contraventionColor;

  ThemeProvider() {
    _loadTheme();
  }

  Future<void> _loadTheme() async {
    final prefs = await SharedPreferences.getInstance();
    _isDarkMode = prefs.getBool('isDarkMode') ?? false;
    _primaryColor = Color(
      prefs.getInt('primaryColor') ??
          (_isDarkMode ? Colors.black.value : Colors.indigo.value),
    );
    _obligatoryColor = Color(
      prefs.getInt('obligatoryColor') ??
          Colors.red.withOpacity(0.5).value,
    );
    _facultativeColor = Color(
      prefs.getInt('facultativeColor') ??
          Colors.amber.withOpacity(0.5).value,
    );
    _nonColor = Color(
      prefs.getInt('nonColor') ??
          Colors.grey.withOpacity(0.5).value,
    );
    _crimeColor = Color(
      prefs.getInt('crimeColor') ??
          Colors.red.withOpacity(0.5).value,
    );
    _delitColor = Color(
      prefs.getInt('delitColor') ??
          Colors.amber.withOpacity(0.5).value,
    );
    _contraventionColor = Color(
      prefs.getInt('contraventionColor') ??
          Colors.green.withOpacity(0.5).value,
    );
    notifyListeners();
  }

  Future<void> toggleTheme(bool isDark) async {
    _isDarkMode = isDark;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setBool('isDarkMode', _isDarkMode);
    notifyListeners();
  }

  Future<void> updatePrimaryColor(Color newColor) async {
    _primaryColor = newColor;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt('primaryColor', newColor.value);
    notifyListeners();
  }

  Future<void> updateObligatoryColor(Color newColor) async {
    _obligatoryColor = newColor;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt('obligatoryColor', newColor.value);
    notifyListeners();
  }

  Future<void> updateFacultativeColor(Color newColor) async {
    _facultativeColor = newColor;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt('facultativeColor', newColor.value);
    notifyListeners();
  }

  Future<void> updateNonColor(Color newColor) async {
    _nonColor = newColor;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt('nonColor', newColor.value);
    notifyListeners();
  }

  Future<void> updateCrimeColor(Color newColor) async {
    _crimeColor = newColor;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt('crimeColor', newColor.value);
    notifyListeners();
  }

  Future<void> updateDelitColor(Color newColor) async {
    _delitColor = newColor;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt('delitColor', newColor.value);
    notifyListeners();
  }

  Future<void> updateContraventionColor(Color newColor) async {
    _contraventionColor = newColor;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt('contraventionColor', newColor.value);
    notifyListeners();
  }

  Future<void> resetInfractionColors() async {
    updateObligatoryColor(Colors.red.withOpacity(0.5));
    updateFacultativeColor(Colors.amber.withOpacity(0.5));
    updateNonColor(Colors.grey.withOpacity(0.5));
    updateCrimeColor(Colors.red.withOpacity(0.5));
    updateDelitColor(Colors.amber.withOpacity(0.5));
    updateContraventionColor(Colors.green.withOpacity(0.5));
  }
}